@@ -40,13 +40,13 @@
 #include "lttng-events.h"
 
 #define LTTNG_MODULES_MAJOR_VERSION 2
-#define LTTNG_MODULES_MINOR_VERSION 6
-#define LTTNG_MODULES_PATCHLEVEL_VERSION 2
+#define LTTNG_MODULES_MINOR_VERSION 7
+#define LTTNG_MODULES_PATCHLEVEL_VERSION 0
 #define LTTNG_MODULES_EXTRAVERSION ""
 
-#define LTTNG_VERSION_NAME		"Gaia"
+#define LTTNG_VERSION_NAME		"Herbe à Détourne"
 #define LTTNG_VERSION_DESCRIPTION	\
-	"From Benelux brewery in Montreal, the Gaia is a witbier with a cloudy light amber ale color. Aroma of coriander with floral hay notes is enchanting. Taste is also dominated by the very floral coriander note and some medium sweet malt notes."
+	"Brewed with unrestrained amounts of Citra hop, the Herbe à Détourne is a fantastic New World Tripel brewed by \"Dieu du Ciel!\". Aromas of mango, cantaloupe melon and passion fruit, combined with a controlled bitter finish, unite in making this smooth golden-orange beer stand apart."
 
 #ifndef CHAR_BIT
 #define CHAR_BIT 8
